!-------------------------------------------------------------------------%
!                       VERTICAL RISER-SOIL MODEL (AUBENY-d)
!-------------------------------------------------------------------------%
!Penetration, z, positive downwards

subroutine aubeny(z, lsnum, LPU, IOP, input_array, work_array, res_array)
    implicit none
	
	!Declare input parameters
	integer :: IOP, LPU, lsnum
	double precision,intent(in), dimension(12) :: input_array	
	double precision :: z 
	double precision :: ks, d, strength_mudline, strength_grad, a_back, b_back, alpha_deg, beta_deg
	double precision :: k0, w_soil , phi_soil, psi_soil 
	
	!Declare intermediate parameters
	integer :: i, lstart, path, path_dir
	double precision, intent(inout), dimension(25) :: work_array
	double precision ::  z_prev, z_1bound, z_2bound, z_3bound, z_lastRev, z_cubicRev
	double precision :: z_1deg, z_1controlDeg, z_pmax, z_2boundc, z_3boundc
	double precision :: p_prev, p_1bound, p_2bound, p_3bound, p_lastRev, p_1controlDeg, p_2boundc, p_cubicRev, deltap_backbone, lambda 
	
	!Declare internal variables
	integer :: ierr, lastpath_dir	
	double precision :: dz, p_bound, p_path12, p_path12c, p_path23
	double precision :: p_reloadcubic, kz_reloadcubic, p_reloadHyperbolic, kz_unloadHyperbolic
	double precision :: p_unloadHyperbolic, kz_reloadHyperbolic, iterationCubic, iterationHyperbolic
	
	!Declare output
	double precision, intent(out), dimension(2) :: res_array
	double precision :: p, kz
	
	!Input parameters
	d = input_array(1)					!Riser diameter
	phi_soil = input_array(2)			!Unload tension limit
	strength_mudline = input_array(3)	!Shear strength at mudline
	strength_grad = input_array(4)		!Shear strength gradient
	ks = input_array(5)					!Elastic stiffness
	k0 = input_array(6) 				!k0=660*strength_mudline
	w_soil = input_array(7)				!Unload large deflections
	psi_soil = input_array(8)			!Riser-soil separation parameter
	a_back = input_array(9)				!Backbone curve coefficient
	b_back = input_array(10)			!backbone curve exponent
	alpha_deg = input_array(11)			!Degradation parameter for calculation of new z_1bound 
	beta_deg = input_array(12)			!Degradation parameter for calculation of new z_1bound 

	!Internal variables
	path = int(work_array(1))			!Path identifier			
	i = int(work_array(2))				!Local load step counter when activating the nonlinear model
	lstart = int(work_array(3))			!Load step nr. when nonlinear model is a activated
	path_dir = int(work_array(4)) 		! -1 for unloading and 1 for loading
	z_prev = work_array(5)				!Previous deflection
	z_1bound = work_array(6)			!Point on backbone curve
	z_2bound = work_array(7)			!Deflection at maximum suction
	z_3bound = work_array(8)			!Deflection at full separation
	z_lastRev = work_array(9)			!Last arbitrary reversal
	z_cubicRev = work_array(10)			!Deflection reversal on partial soil-separation curve 
	z_1deg = work_array(11)				!Deflection at initial penetration due to selfweight
	z_1controlDeg = work_array(12)		!Degradation control parameter
	z_pmax = work_array(13) 			!Control parameter at maximum soil resistance
	z_2boundc  = work_array(14)			!Control parameter z_2bound by use of z_pmax 
	z_3boundc = work_array(15)			!Control parameter z_3bound by use of z_pmax
	p_prev = work_array(16)				!Soil resistance at previous load step
	p_1bound = work_array(17)			!Maximum soil resistance
	p_2bound = work_array(18)			!Maximum soil suction
	p_3bound = work_array(19)           !Soil resistance at full separation							
	p_lastRev = work_array(20)			!Soil resistance at last arbitrary reversal
	p_cubicRev = work_array(21) 		!Soil resistance at corresponding z_cubicRev
	p_1controlDeg = work_array(22)		!Degradation control parameter
	p_2boundc = work_array(23)			!Control parameter p_2bound by use of z_pmax	
	deltap_backbone = work_array(24)	!Load correction of backbone curve due to degradation
	lambda = work_array(25)				!Accumulated displacement		
	
		
	select case(IOP)
		case(1) !Initial linear springs		
		p = ks*z
		kz = ks
		path = 100
		!No suction is considered when loss of contact
		if (z<0.0d0) then
			path = 101   
		end if
	
		case(2)	 !Nonlinear Soil Model		
		if (i<=0) then
			!Initial calculation, there is no static approximation
			p_prev = a_back*(z_prev/d)**b_back*(strength_mudline+strength_grad*z_prev)*d 
			lstart = lsnum
			path = 0 			
		end if

		!Check transition from linear to non-linear
		if (i==0) then
			if (z < 0.0d0) then
				write(*,*) 'There is no soil contact, try reducing the starting time'
				write(LPU,*) 'There is no soil contact, try reducing the starting time'
				ierr = ierr-1 
				STOP
			end if
		end if
	
		!Load step
		i = lsnum - (lstart-1)	
				
		lastpath_dir = path_dir
		if (z>z_prev)then
			path_dir = 1
		elseif (lastpath_dir == 1 .and. z==z_prev) then
			path_dir = 1
		else
			path_dir = -1
		end if		
		
!---------------------------------- Define Degradation Control Point ---------------------------------		
		!Define reversals
		if (path_dir /= lastpath_dir) then 
			z_lastrev = z_prev
			p_lastRev = p_prev			
		end if
		
		!Reset degradation rule when z_1controlDeg is exceeded
		if (z>z_1controlDeg .and. z<z_prev) then 
			z_1deg= z_prev
			lambda = 0.0d0
		end if	
	
		if (z>=z_3bound .and. path_dir == -1) then 
			dz = abs(z-z_prev) 
		elseif (z<z_3bound .and. z_3bound<z_prev) then     
			dz = abs(z_3bound-z_prev)
		end if
		lambda = lambda + dz	
		z_1controlDeg = z_1deg + alpha_deg*(lambda)**beta_deg 
		p_1controlDeg = a_back*(z_1controlDeg/d)**b_back*(strength_mudline+strength_grad*z_1controlDeg)*d - deltap_backbone 	
		
!---------------------------- Calculation of Soil Resistance and Stiffness ---------------------------	
		!Backbone curve
		!Define characteristic along bounding loop
10		if (path==0) then 	
			p = a_back*(z/d)**b_back*(strength_mudline + strength_grad*z)*d
			p = p-deltap_backbone
			kz = a_back*b_back *(strength_mudline + strength_grad*z)*(z/d)**(b_back-1) + d*a_back*strength_grad*(z/d)**b_back			
			if (z<z_prev .and. i>=1) then            
				z_1bound = z_prev		
				p_1bound = p_prev
				call revboundingloop(z_1bound, p_1bound, phi_soil, k0, w_soil, psi_soil, z_2bound, p_2bound, z_3bound)
				if (z<z_2bound .and. z>z_3bound) then		
					path = 23  
					go to 2
				elseif (z<z_3bound) then
					path = 30   
					go to 3
				else 
					path = 12   
					go to 1
				end if
			end if
		end if
			
	!Path 1-2; Elastic rebound		
1	if (path == 12) then
		p = p_unloadHyperbolic(z, path_dir, z_1bound, p_1bound, p_1bound, k0, w_soil)
		kz = kz_unloadHyperbolic(z, path_dir, z_1bound, p_1bound, k0, w_soil)
		if (z <= z_2bound) then	
			path = 23  
			go to 2
		end if
		if (z>z_prev) then !Arbitrary reversal point on path 1-2
		path = 121   
			go to 6
		end if
	end if
		
	!Path 2-3; Partial soil-separation 
2	if (path == 23) then
		p = p_reloadcubic(z, z_2bound, z_3bound, p_2bound, p_3bound)
		kz = kz_reloadcubic(z, z_2bound, z_3bound, p_2bound, p_3bound)
		if (z > z_prev) then
			z_cubicRev = z_prev
			p_cubicRev = p_prev
			path = 321  
			go to 4
		end if
		if (z<z_3bound) then
			path = 30
			go to 3
		end if
	end if
	
	!Path 3-0; Full separation
3	if (path == 30) then 
		p = 0.0d0
		kz = 0.0d0		
		if (z>=z_3bound) then 
			z_cubicRev = z_3bound
			p_cubicRev = 0.0d0
			path = 321 
			go to 4
		end if
	end if	
	
	!Path 3-2-1; Re-loading after full separation and reversal from partial soil separation curve
4	if (path==321) then
		p = p_reloadcubic(z, z_1controlDeg, z_cubicRev, p_1controlDeg, p_cubicRev)
		kz = kz_reloadcubic(z, z_1controlDeg, z_cubicRev, p_1controlDeg, p_cubicRev)
		if (z >= z_1controlDeg) then
			deltap_backbone = deltap_backbone + (p_1controlDeg-p_1bound)
			path = 0
			go to 10
		end if
		if (p>p_1bound) then
			p = p_1bound
			kz = 0.0d0
		end if
		if (z<z_prev) then		
			if (p_lastRev>=p_1bound) then
				z_1bound = z_lastRev
				p_1bound = p_lastRev	
				call revboundingloop(z_1bound, p_1bound, phi_soil, k0, w_soil, psi_soil, z_2bound, p_2bound, z_3bound)	
				path = 12
				go to 1
			end if				
			z_pmax =  iterationCubic(z_1controlDeg, z_3bound, p_1controlDeg, p_3bound, p_1bound)
			call revboundingloop(z_pmax, p_1bound, phi_soil, k0, w_soil, psi_soil, z_2boundc, p_2boundc, z_3boundc)
			path = 122		
			go to 5
		end if			
	end	if	
	
	!Path 1-2-2; Unloading from path 3-1 or from an arbitrary reversal within bounding loop
5	if 	(path == 122) then
		p = p_unloadHyperbolic(z, path_dir, z_lastRev, p_lastRev, p_1bound, k0, w_soil) 
		kz = kz_unloadHyperbolic(z, path_dir, z_lastRev, p_1bound, k0, w_soil)
		if (z>z_prev) then		
			path = 121
			go to 6
		else
			p_path12c  = p_unloadHyperbolic(z_lastRev, -1, z_1bound, p_1bound, p_1bound, k0, w_soil)	
			if (z_2bound<z .and. z_lastRev<=z_1bound .and. p_lastRev>=p_path12c) then
			p_path12  = p_unloadHyperbolic(z, -1, z_1bound, p_1bound, p_1bound, k0, w_soil)
				if (p<=p_path12) then					
					path = 12
					go to 1
				end if
			elseif (z_3bound<z .and. z<=z_2bound .and. z_lastRev<z_1bound .and. p_lastRev>=p_path12c) then
				p_path23 = p_reloadcubic(z, z_2bound, z_3bound, p_2bound, p_3bound)					
				if (p<=p_path23) then
					path = 23
					go to 2
				end if	
			elseif (z_pmax>=z .and. z>z_2boundc) then				
				p_path12 = p_unloadHyperbolic(z, -1, z_pmax, p_1bound, p_1bound, k0, w_soil)					
				if (p<=p_path12) then
					p = p_unloadHyperbolic(z, -1, z_pmax, p_1bound, p_1bound, k0, w_soil)	
				end if
			elseif(z_2boundc> z.and. z>z_3boundc) then
				p_path23 = p_reloadcubic(z, z_2boundc, z_3boundc, p_2boundc, p_3bound)				
				if (p<=p_path23) then
					p = p_reloadcubic(z, z_2boundc, z_3boundc, p_2boundc, p_3bound)
					call revboundingloop(z_pmax, p_1bound, phi_soil, k0, w_soil, psi_soil, z_2bound, p_2bound, z_3bound)	
				end if	
			elseif (z<=z_3bound) then
				path = 30
				go to 3
			end if
		end	 if	
	end if	

	!Path 1-2-1; Reloading from path 2-3, 1-2 or from an arbitrary reversal within bounding loop
6	if (path == 121) then	
		if (z>=z_prev) then		
			if (z >= z_1controlDeg) then
				deltap_backbone = deltap_backbone + (p_1controlDeg- p_1bound)
				path = 0
				go to 10
			end if	
			p = p_reloadHyperbolic(z, path_dir, z_lastRev, z_1controlDeg, p_lastRev, p_1controlDeg, k0, w_soil)
			kz = kz_reloadHyperbolic(z, path_dir, z_lastRev, z_1controlDeg, p_lastRev, p_1controlDeg, k0, w_soil)
			p_bound = p_reloadcubic(z, z_1controlDeg, z_cubicRev, p_1controlDeg, p_cubicRev)
			if (p>=p_bound) then
				path = 321 
				go to 4				
			end if
			if (p_lastRev == p_prev) then
				z_pmax =  iterationHyperbolic(z_1controlDeg, z_lastRev, p_1controlDeg, p_lastRev, p_1bound, k0, w_soil)	
			end if
			if (p>p_1bound) then
				p=p_1bound
				kz = 0.0d0
			end if				
		else
		if (p_prev>=p_1bound) then	
			z_1bound = z_lastRev
			p_1bound = p_lastRev
			call revboundingloop(z_1bound, p_1bound, phi_soil, k0, w_soil, psi_soil, z_2bound, p_2bound, z_3bound)	
			path = 12
			go to 1		
		end	if
		call revboundingloop(z_pmax, p_1bound, phi_soil, k0, w_soil, psi_soil, z_2boundc, p_2boundc, z_3boundc)	
		path = 122		
		go to 5
		end if
	end if	

!-----------------------------------------------Updating parameters-----------------------------------
		work_array(1) = real(path) 
		work_array(2) = real(i)
		work_array(3) = real(lstart)		
		work_array(4) = real(path_dir)
		work_array(5) = z  !z_prev
		work_array(6) = z_1bound	
		work_array(7) = z_2bound
		work_array(8) = z_3bound
		work_array(9) = z_lastRev
		work_array(10) = z_cubicRev 
		work_array(11) = z_1deg	
		work_array(12) = z_1controlDeg 
		work_array(13) = z_pmax
		work_array(14) = z_2boundc  
		work_array(15) = z_3boundc
		work_array(16) = p !p_prev
		work_array(17) = p_1bound
		work_array(18) = p_2bound
		work_array(19) = p_3bound 
		work_array(20) = p_lastRev	
		work_array(21) = p_cubicRev 
		work_array(22) = p_1controlDeg 
		work_array(23) = p_2boundc 	
		work_array(24) = deltap_backbone	
		work_array(25) = lambda			
		
	end select	
	
!---------------------------------------- Updating result array --------------------------------------
	res_array(1:2)=(/p,kz/)
	
end subroutine aubeny
