%-------------------------------------------------------------------------%
%                       VERTICAL RISER-SOIL MODEL (AUBENY-d)
%-------------------------------------------------------------------------%
%Penetration, z, positive downwards

function [work_array, res] = aubeny(z, lsnum, LPU, IOP, input_array, work_array, res)
    %Input parameters
    d = input_array(1);					%Riser diameter
	phi_soil = input_array(2);			%Unload tension limit
	strength_mudline = input_array(3);	%Shear strength at mudline
	strength_grad = input_array(4);		%Shear strength gradient
	ks = input_array(5);				%Elastic stiffness
	k0 = input_array(6); 				%k0=660*strength_mudline
	w_soil = input_array(7);			%Unload large deflections
	psi_soil = input_array(8);			%Riser-soil separation parameter
	a_back = input_array(9);			%Backbone curve coefficient
	b_back = input_array(10);			%backbone curve exponent
	alpha_deg = input_array(11);		%Degradation parameter for calculation of new z_1bound 
	beta_deg = input_array(12);			%Degradation parameter for calculation of new z_1bound 

    %Intermediate variables
	path = work_array(1);               %Path identifier			
	i = work_array(2);                  %Local load step counter is activated for the nonlinear model
	lstart = work_array(3);             %Load step nr. when nonlinear model is a activated
	path_dir = work_array(4);           % -1 for unloading and 1 for loading
	z_prev = work_array(5);             %Previous deflection
	%z_1bound = work_array(6);          %Point on backbone curve
	%z_2bound = work_array(7);          %Deflection at maximum suction
	%z_3bound = work_array(8);          %Deflection at full separation
	%z_lastRev = work_array(9);         %Last arbitrary reversal
	%z_cubicRev = work_array(10);       %Deflection reversal on partial soil-separation curve 
	z_1deg = work_array(11);            %Deflection at initial penetration due to selfweight
	%z_1controlDeg = work_array(12);    %Degradation control parameter
	%z_pmax = work_array(13);           %Control parameter at maximum soil resistance
	%z_2boundc  = work_array(14);		%Control parameter z_2bound by use of z_pmax 
	%z_3boundc = work_array(15);		%Control parameter z_3bound by use of z_pmax
	p_prev = work_array(16);			%Soil resistance at previous load step
	%p_1bound = work_array(17);			%Maximum soil resistance
	%p_2bound = work_array(18);			%Maximum soil suction			
	%p_lastRev = work_array(19);		%Soil resistance at last arbitrary reversal
	%p_cubicRev = work_array(20); 		%Soil resistance at corresponding z_cubicRev
	%p_1controlDeg = work_array(21);	%Degradation control parameter
	%p_2boundc = work_array(22);		%Control parameter p_2bound by use of z_pmax	
	%dp_backbone = work_array(23);      %Load correction of backbone curve due to degradation
	lambda = work_array(24);			%Accumulated displacement	
    
    dz = 0;   
    switch IOP
        case 1  %Initial linear springs		
        p = ks*z;
        kz = ks;
        path = 100;
        %No suction is considered when loss of contact
        if z<0 
        	path = 101;  
        end 
        
        case 2  %Nonlinear Soil Model
        if i<=0 
			%Initial calculation, there is no static approximation
			p_prev = a_back*(z_prev/d)^b_back*(strength_mudline+strength_grad*z_prev)*d; 
			lstart = lsnum;
			path = 0; 			
        end 
 
		%Check transition from linear to non-linear
        if i == 0
            if z<0 
                disp('There is no soil contact, try reducing the starting time')
                fprintf(LPU,'There is no soil contact, try reducing the starting time');
                return
            end
        end	
   	 
		i = lsnum - (lstart-1);	
		%Define unloading or re-loading		
		lastpath_dir = path_dir;
		if z>z_prev
            path_dir = 1;
		elseif lastpath_dir == 1 && z==z_prev
            path_dir = 1;
		else
			path_dir = -1;
        end 
        
		%Define reversals
        if path_dir ~= lastpath_dir  
			work_array(9) = z_prev;
			work_array(19)	= p_prev;			
        end         
%--------------------------------- Define Degradation Control Point -------------------------------%			   
             
        %Reset degradation rule when z_1controlDeg is exceeded
		if z_prev>work_array(12) && z<z_prev 
			z_1deg= z_prev;
			lambda = 0;
		end 
	
		if z>=work_array(8) && path_dir == -1
			dz = abs(z-z_prev); 
		elseif z<work_array(8) && work_array(8)<z_prev  
			dz = abs(work_array(8)-z_prev);
		end 
		lambda = lambda + dz;	
		work_array(12)= z_1deg + alpha_deg*(lambda)^beta_deg;
		work_array(21) = a_back*(work_array(12)/d)^b_back*(strength_mudline+strength_grad*work_array(12))*d - work_array(23); 	    
 
%----------------------------- Calculation of Soil Resistance and Stiffness ------------------------     
        %Backbone curve
		%Define characteristic along bounding loop
		if path == 0        
			[p, kz, path, work_array] = path01(z, d, a_back, b_back, strength_mudline, strength_grad, work_array);            
			if z<z_prev && i>=1                  
                work_array(6) = z_prev;		
				work_array(17) = p_prev; 
                %Define characteristic point on bounding loop
                [work_array(7), work_array(18), work_array(8)] = revboundingloop(work_array(6),work_array(17), phi_soil, k0, w_soil, psi_soil);
				if z<work_array(7) && z>work_array(8)	                
					[p, kz, path, work_array] = path23(z, work_array, d, phi_soil, strength_mudline,strength_grad, k0, w_soil, psi_soil, a_back,b_back);
				elseif z<work_array(8)                    
                  	[p, kz, path, work_array]  = path30(z, work_array, d, phi_soil, strength_mudline,strength_grad, k0, w_soil, psi_soil, a_back,b_back);
				else                     
                    [p, kz, path, work_array]  =  path12(z, work_array, d, phi_soil, strength_mudline,strength_grad, k0, w_soil, psi_soil, a_back,b_back);                   
				end 
			end 
        end 
     
%----------------------------------Continiue on the same path--------------------------------------%
if path == 12   
    [p, kz, path, work_array]  =  path12(z, work_array, d, phi_soil, strength_mudline,strength_grad, k0, w_soil, psi_soil, a_back,b_back);
 end

if path == 121
   [p, kz, path, work_array]  = path121(z, work_array, d, phi_soil, strength_mudline,strength_grad, k0, w_soil, psi_soil, a_back,b_back);
end

if path == 12
    [p, kz, path, work_array]  = path122(z, work_array, d, phi_soil, strength_mudline,strength_grad, k0, w_soil, psi_soil, a_back,b_back);
end

if path == 23
    [p, kz, path, work_array]  = path23(z, work_array, d, phi_soil, strength_mudline,strength_grad, k0, w_soil, psi_soil, a_back,b_back);
end

if path == 321
    [p, kz, path, work_array]  = path321(z, work_array, d, phi_soil, strength_mudline,strength_grad, k0, w_soil, psi_soil, a_back,b_back);
end

if path == 30
    [p, kz, path, work_array]  = path30(z, work_array, d, phi_soil, strength_mudline,strength_grad, k0, w_soil, psi_soil, a_back,b_back);
end

%----------------------------------Updating some parameters-----------------------------------------
		
        work_array(1) = path; 
		work_array(2) = i;
		work_array(3) = lstart;		
		work_array(4) = path_dir;
		work_array(5) = z;  %z_prev
		work_array(11) = z_1deg;	
		work_array(16) = p; %p_prev
		work_array(24) = lambda;	
 
%-------------------------------- Updating result array ---------------------------------------------        
        res(lsnum,1) = kz;
        res(lsnum,2) = p;  
    end     
end 
